@@ -37,6 +37,11 @@
 
 namespace VeraCrypt
 {
+#ifdef TC_MACOSX
+	int GraphicUserInterface::g_customIdCmdV = 0;
+	int GraphicUserInterface::g_customIdCmdA = 0;
+#endif
+
 	GraphicUserInterface::GraphicUserInterface () :
 		ActiveFrame (nullptr),
 		BackgroundMode (false),
@@ -51,6 +56,8 @@ namespace VeraCrypt
 #endif
 
 #ifdef TC_MACOSX
+		g_customIdCmdV = wxNewId();
+		g_customIdCmdA = wxNewId();
 		wxApp::s_macHelpMenuTitleName = _("&Help");
 #endif
 	}
@@ -201,7 +208,42 @@ namespace VeraCrypt
 				}
 				catch (PasswordException &e)
 				{
-					ShowWarning (e);
+					bool bFailed = true;
+					if (!options->UseBackupHeaders)
+					{
+						try
+						{
+							OpenVolumeThreadRoutine routine2(
+								options->Path,
+								options->PreserveTimestamps,
+								options->Password,
+								options->Pim,
+								options->Kdf,
+								false,
+								options->Keyfiles,
+								options->Protection,
+								options->ProtectionPassword,
+								options->ProtectionPim,
+								options->ProtectionKdf,
+								options->ProtectionKeyfiles,
+								true,
+								volumeType,
+								true
+							);
+
+							ExecuteWaitThreadRoutine (parent, &routine2);
+							volume = routine2.m_pVolume;
+							bFailed = false;
+						}
+						catch (...)
+						{
+						}
+					}
+
+					if (bFailed)
+						ShowWarning (e);
+					else
+						ShowWarning ("HEADER_DAMAGED_AUTO_USED_HEADER_BAK");
 				}
 			}
 
@@ -219,7 +261,7 @@ namespace VeraCrypt
 
 				wxSingleChoiceDialog choiceDialog (parent, LangString["DOES_VOLUME_CONTAIN_HIDDEN"], Application::GetName(), choices);
 				choiceDialog.SetSize (wxSize (Gui->GetCharWidth (&choiceDialog) * 60, -1));
-				choiceDialog.SetSelection (-1);
+				choiceDialog.SetSelection (0);
 
 				if (choiceDialog.ShowModal() != wxID_OK)
 					return;
@@ -657,6 +699,41 @@ namespace VeraCrypt
 	{
 		SetBackgroundMode (false);
 	}
+	
+	bool GraphicUserInterface::HandlePasswordEntryCustomEvent (wxEvent& event)
+	{
+		bool bHandled = false;
+		if (	(event.GetEventType() == wxEVT_MENU)
+			&&	((event.GetId() == g_customIdCmdV) || (event.GetId() == g_customIdCmdA)))
+		{
+			wxWindow* focusedCtrl = wxWindow::FindFocus();
+			if (focusedCtrl 
+				&& (focusedCtrl->IsKindOf(wxCLASSINFO(wxTextCtrl)))
+				&& (focusedCtrl->GetWindowStyle() & wxTE_PASSWORD))
+			{
+				wxTextCtrl* passwordCtrl = (wxTextCtrl*) focusedCtrl;
+				if (event.GetId() == g_customIdCmdV)
+					passwordCtrl->Paste ();
+				else if (event.GetId() == g_customIdCmdA)
+					passwordCtrl->SelectAll ();
+				bHandled = true;
+			}
+		}
+		
+		return bHandled;
+	}
+	
+	void GraphicUserInterface::InstallPasswordEntryCustomKeyboardShortcuts (wxWindow* window)
+	{
+		// we manually handle CMD+V and CMD+A on password fields in order to support
+		// pasting password values into them. By default, wxWidgets doesn't handle this
+		// for password entry fields.
+		wxAcceleratorEntry entries[2];
+		entries[0].Set(wxACCEL_CMD, (int) 'V', g_customIdCmdV);
+		entries[1].Set(wxACCEL_CMD, (int) 'A', g_customIdCmdA);
+		wxAcceleratorTable accel(sizeof(entries) / sizeof(wxAcceleratorEntry), entries);
+		window->SetAcceleratorTable(accel);
+	}
 #endif
 
 	void GraphicUserInterface::MoveListCtrlItem (wxListCtrl *listCtrl, long itemIndex, long newItemIndex) const
@@ -1247,9 +1324,13 @@ namespace VeraCrypt
 			if (!localFile)
 			{
 				htmlPath = L"https://www.veracrypt.fr/en/";
-				url.Replace (L" ", L"%20");
-				url.Replace (L"'", L"%27");
 			}
+			else
+			{
+				htmlPath = L"file://" + htmlPath;
+			}
+			url.Replace (L" ", L"%20");
+			url.Replace (L"'", L"%27");
 
 			url = htmlPath + url;
 		}
@@ -1320,7 +1401,7 @@ namespace VeraCrypt
 
 		wxSingleChoiceDialog choiceDialog (parent, LangString["HEADER_RESTORE_EXTERNAL_INTERNAL"], Application::GetName(), choices);
 		choiceDialog.SetSize (wxSize (Gui->GetCharWidth (&choiceDialog) * 80, -1));
-		choiceDialog.SetSelection (-1);
+		choiceDialog.SetSelection (0);
 
 		if (choiceDialog.ShowModal() != wxID_OK)
 			return;
@@ -1569,13 +1650,18 @@ namespace VeraCrypt
 
 	DirectoryPath GraphicUserInterface::SelectDirectory (wxWindow *parent, const wxString &message, bool existingOnly) const
 	{
+		/* Avoid OS leaking previously used directory when user choose not to save history */
+		wxString defaultPath;
+		if (!GetPreferences().SaveHistory)
+			defaultPath = wxGetHomeDir ();
+
 		return DirectoryPath (::wxDirSelector (!message.empty() ? message :
 #ifdef __WXGTK__
 			wxDirSelectorPromptStr,
 #else
 			L"",
 #endif
-			L"", wxDD_DEFAULT_STYLE | (existingOnly ? wxDD_DIR_MUST_EXIST : 0), wxDefaultPosition, parent).wc_str());
+			defaultPath, wxDD_DEFAULT_STYLE | (existingOnly ? wxDD_DIR_MUST_EXIST : 0), wxDefaultPosition, parent).wc_str());
 	}
 
 	FilePathList GraphicUserInterface::SelectFiles (wxWindow *parent, const wxString &caption, bool saveMode, bool allowMultiple, const list < pair <wstring, wstring> > &fileExtensions, const DirectoryPath &directory) const
@@ -1614,7 +1700,12 @@ namespace VeraCrypt
 			}
 		}
 
-		wxFileDialog dialog (parent, !caption.empty() ? caption : LangString ["OPEN_TITLE"], wstring (directory), wxString(), wildcards, style);
+		/* Avoid OS leaking previously used directory when user choose not to save history */
+		wxString defaultDir = wstring (directory);
+		if (defaultDir.IsEmpty () && !GetPreferences().SaveHistory)
+			defaultDir = wxGetHomeDir ();
+
+		wxFileDialog dialog (parent, !caption.empty() ? caption : LangString ["OPEN_TITLE"], defaultDir, wxString(), wildcards, style);
 
 		if (dialog.ShowModal() == wxID_OK)
 		{
@@ -1636,7 +1727,7 @@ namespace VeraCrypt
 	FilePath GraphicUserInterface::SelectVolumeFile (wxWindow *parent, bool saveMode, const DirectoryPath &directory) const
 	{
 		list < pair <wstring, wstring> > extensions;
-		extensions.push_back (make_pair (L"tc", LangString["TC_VOLUMES"].ToStdWstring()));
+		extensions.push_back (make_pair (L"hc", LangString["TC_VOLUMES"].ToStdWstring()));
 
 		FilePathList selFiles = Gui->SelectFiles (parent, LangString[saveMode ? "OPEN_NEW_VOLUME" : "OPEN_VOL_TITLE"], saveMode, false, extensions, directory);
 
