@@ -13,6 +13,7 @@
 
 #include "Tcdefs.h"
 #include <Shlobj.h>
+#include <Richedit.h>
 #include <io.h>
 #include <stdio.h>
 #include <time.h>
@@ -23,6 +24,7 @@
 #include "Common/Resource.h"
 #include "Resource.h"
 #include "Setup.h"
+#include "Registry.h"
 #include <tchar.h>
 #include <Strsafe.h>
 
@@ -31,9 +33,11 @@ using namespace std;
 enum wizard_pages
 {
 	INTRO_PAGE,
+#ifndef PORTABLE
 	WIZARD_MODE_PAGE,
 	INSTALL_OPTIONS_PAGE,
 	INSTALL_PROGRESS_PAGE,
+#endif
 	EXTRACTION_OPTIONS_PAGE,
 	EXTRACTION_PROGRESS_PAGE,
 	DONATIONS_PAGE
@@ -57,6 +61,8 @@ BOOL bInProgress = FALSE;
 BOOL bPromptTutorial = FALSE;
 BOOL bPromptReleaseNotes = FALSE;
 
+extern BOOL bUserSetLanguage;
+
 int nPbar = 0;			/* Control ID of progress bar */
 
 static HFONT hDonTextFont;
@@ -125,6 +131,7 @@ void LoadPage (HWND hwndDlg, int nPageNo)
 					 (DLGPROC) PageDialogProc);
 		break;
 
+#ifndef PORTABLE
 	case WIZARD_MODE_PAGE:
 		hCurPage = CreateDialogW (hInst, MAKEINTRESOURCEW (IDD_WIZARD_MODE_PAGE_DLG), hwndDlg,
 					 (DLGPROC) PageDialogProc);
@@ -139,6 +146,7 @@ void LoadPage (HWND hwndDlg, int nPageNo)
 		hCurPage = CreateDialogW (hInst, MAKEINTRESOURCEW (IDD_PROGRESS_PAGE_DLG), hwndDlg,
 					 (DLGPROC) PageDialogProc);
 		break;
+#endif
 
 	case EXTRACTION_OPTIONS_PAGE:
 		hCurPage = CreateDialogW (hInst, MAKEINTRESOURCEW (IDD_EXTRACTION_OPTIONS_PAGE_DLG), hwndDlg,
@@ -222,10 +230,24 @@ BOOL CALLBACK PageDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 			{
 				char *licenseText = NULL;
 
+#ifdef PORTABLE
+				bExtractOnly = TRUE;
+#endif
+
+				// increase size limit of rich edit control
+				SendMessage (GetDlgItem (hwndDlg, IDC_LICENSE_TEXT), EM_EXLIMITTEXT, 0, -1);
+				// Left margin for license text
+				SendMessage (GetDlgItem (hwndDlg, IDC_LICENSE_TEXT), EM_SETMARGINS, (WPARAM) EC_LEFTMARGIN, (LPARAM) CompensateXDPI (4));
+
 				licenseText = GetLegalNotices ();
 				if (licenseText != NULL)
 				{
-					SetWindowTextA (GetDlgItem (hwndDlg, IDC_LICENSE_TEXT), licenseText);
+					SETTEXTEX TextInfo = {0};
+
+					TextInfo.flags = ST_SELECTION;
+					TextInfo.codepage = CP_ACP;
+
+					SendMessage(GetDlgItem (hwndDlg, IDC_LICENSE_TEXT), EM_SETTEXTEX, (WPARAM)&TextInfo, (LPARAM)licenseText);
 					free (licenseText);
 				}
 				else
@@ -258,12 +280,10 @@ BOOL CALLBACK PageDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 				EnableWindow (GetDlgItem (GetParent (hwndDlg), IDC_NEXT), bLicenseAccepted);
 				EnableWindow (GetDlgItem (GetParent (hwndDlg), IDC_PREV), FALSE);
 				EnableWindow (GetDlgItem (GetParent (hwndDlg), IDHELP), bLicenseAccepted);
-
-				// Left margin for license text
-				SendMessage (GetDlgItem (hwndDlg, IDC_LICENSE_TEXT), EM_SETMARGINS, (WPARAM) EC_LEFTMARGIN, (LPARAM) CompensateXDPI (4));
 			}
 			return 1;
 
+#ifndef PORTABLE
 		case WIZARD_MODE_PAGE:
 			{
 				LONG driverVersion;
@@ -302,6 +322,7 @@ BOOL CALLBACK PageDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 				EnableWindow (GetDlgItem (GetParent (hwndDlg), IDC_PREV), TRUE);
 			}
 			return 1;
+#endif
 
 		case EXTRACTION_OPTIONS_PAGE:
 
@@ -374,6 +395,7 @@ BOOL CALLBACK PageDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 
 			return 1;
 
+#ifndef PORTABLE
 		case INSTALL_OPTIONS_PAGE:
 			{
 				LONG driverVersion;
@@ -477,6 +499,7 @@ BOOL CALLBACK PageDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 			}
 
 			return 1;
+#endif
 
 		case DONATIONS_PAGE:
 
@@ -592,7 +615,7 @@ BOOL CALLBACK PageDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 			EnableWindow (GetDlgItem (GetParent (hwndDlg), IDC_NEXT), IsButtonChecked (GetDlgItem (hwndDlg, IDC_AGREE)));
 			return 1;
 		}
-
+#ifndef PORTABLE
 		if (lw == IDC_WIZARD_MODE_EXTRACT_ONLY && nCurPageNo == WIZARD_MODE_PAGE)
 		{
 			bExtractOnly = TRUE;
@@ -604,19 +627,19 @@ BOOL CALLBACK PageDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 			bExtractOnly = FALSE;
 			return 1;
 		}
-
+#endif
 		if ( nCurPageNo == EXTRACTION_OPTIONS_PAGE && hw == EN_CHANGE )
 		{
 			EnableWindow (GetDlgItem (MainDlg, IDC_NEXT), (GetWindowTextLength (GetDlgItem (hCurPage, IDC_DESTINATION)) > 1));
 			return 1;
 		}
-
+#ifndef PORTABLE
 		if ( nCurPageNo == INSTALL_OPTIONS_PAGE && hw == EN_CHANGE )
 		{
 			EnableWindow (GetDlgItem (MainDlg, IDC_NEXT), (GetWindowTextLength (GetDlgItem (hCurPage, IDC_DESTINATION)) > 1));
 			return 1;
 		}
-
+#endif
 		if ( nCurPageNo == EXTRACTION_OPTIONS_PAGE )
 		{
 			switch (lw)
@@ -637,7 +660,7 @@ BOOL CALLBACK PageDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 				return 1;
 			}
 		}
-
+#ifndef PORTABLE
 		if ( nCurPageNo == INSTALL_OPTIONS_PAGE )
 		{
 			switch (lw)
@@ -675,7 +698,7 @@ BOOL CALLBACK PageDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 
 			}
 		}
-
+#endif
 		if (nCurPageNo == DONATIONS_PAGE)
 		{
 			switch (lw)
@@ -821,8 +844,11 @@ BOOL CALLBACK MainDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 			nPbar = IDC_PROGRESS_BAR;
 
 			SendMessage (GetDlgItem (hwndDlg, IDC_BOX_TITLE), WM_SETFONT, (WPARAM) hUserBoldFont, (LPARAM) TRUE);
-
+#ifndef PORTABLE
 			SetWindowText (hwndDlg, L"VeraCrypt Setup " _T(VERSION_STRING));
+#else
+			SetWindowText (hwndDlg, L"VeraCrypt Portable " _T(VERSION_STRING));
+#endif
 
 			DonColorSchemeId = GetDonVal (2, 9);
 
@@ -835,7 +861,12 @@ BOOL CALLBACK MainDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 				bDesktopIcon = TRUE;
 				bLicenseAccepted = TRUE;
 				bStartInstall = TRUE;
+#ifdef PORTABLE
+				bExtractOnly = TRUE;
+				LoadPage (hwndDlg, EXTRACTION_PROGRESS_PAGE);
+#else
 				LoadPage (hwndDlg, INSTALL_PROGRESS_PAGE);
+#endif
 			}
 			else
 				LoadPage (hwndDlg, INTRO_PAGE);
@@ -901,7 +932,7 @@ BOOL CALLBACK MainDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 					RegCloseKey (hkey);
 				}
 			}
-
+#ifndef PORTABLE
 			else if (nCurPageNo == WIZARD_MODE_PAGE)
 			{
 				if (IsButtonChecked (GetDlgItem (hCurPage, IDC_WIZARD_MODE_EXTRACT_ONLY)))
@@ -918,14 +949,14 @@ BOOL CALLBACK MainDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 					nCurPageNo = EXTRACTION_OPTIONS_PAGE - 1;
 				}
 			}
-
+#endif
 			else if (nCurPageNo == EXTRACTION_OPTIONS_PAGE)
 			{
 				GetWindowText (GetDlgItem (hCurPage, IDC_DESTINATION), WizardDestExtractPath, ARRAYSIZE (WizardDestExtractPath));
 
 				bStartExtraction = TRUE;
 			}
-
+#ifndef PORTABLE
 			else if (nCurPageNo == INSTALL_OPTIONS_PAGE)
 			{
 				GetWindowText (GetDlgItem (hCurPage, IDC_DESTINATION), WizardDestInstallPath, ARRAYSIZE (WizardDestInstallPath));
@@ -938,7 +969,7 @@ BOOL CALLBACK MainDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 				PostMessage (hwndDlg, WM_CLOSE, 0, 0);
 				return 1;
 			}
-
+#endif
 			else if (nCurPageNo == EXTRACTION_PROGRESS_PAGE)
 			{
 				PostMessage (hwndDlg, WM_CLOSE, 0, 0);
@@ -961,22 +992,25 @@ BOOL CALLBACK MainDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 
 		if (lw == IDC_PREV)
 		{
+#ifndef PORTABLE
 			if (nCurPageNo == WIZARD_MODE_PAGE)
 			{
 				bExtractOnly = IsButtonChecked (GetDlgItem (hCurPage, IDC_WIZARD_MODE_EXTRACT_ONLY));
-			}
-
-			else if (nCurPageNo == EXTRACTION_OPTIONS_PAGE)
+			} else
+#endif
+			if (nCurPageNo == EXTRACTION_OPTIONS_PAGE)
 			{
 				GetWindowText (GetDlgItem (hCurPage, IDC_DESTINATION), WizardDestExtractPath, ARRAYSIZE (WizardDestExtractPath));
+#ifndef PORTABLE
 				nCurPageNo = WIZARD_MODE_PAGE + 1;
+#endif
 			}
-
+#ifndef PORTABLE
 			else if (nCurPageNo == INSTALL_OPTIONS_PAGE)
 			{
 				GetWindowText (GetDlgItem (hCurPage, IDC_DESTINATION), WizardDestInstallPath, ARRAYSIZE (WizardDestInstallPath));
 			}
-
+#endif
 			LoadPage (hwndDlg, --nCurPageNo);
 
 			return 1;
@@ -1038,6 +1072,14 @@ BOOL CALLBACK MainDialogProc (HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
 
 		/* Installation completed successfully */
 
+		/* if user selected a language, use for GUI in the next run */
+		if (bUserSetLanguage)
+		{
+			WCHAR langId[6];
+			MultiByteToWideChar (CP_ACP, 0, GetPreferredLangId(), -1, langId, ARRAYSIZE (langId));
+			WriteRegistryString (L"Software\\VeraCrypt", L"SetupUILanguage", langId);
+		}
+
 		bInProgress = FALSE;
 
 		nCurPageNo = DONATIONS_PAGE;
